/***********************************************************************************
 
 This software module was originally developed by
 
 Apple Inc. and Fraunhofer IIS
 
 in the course of development of the ISO/IEC 23003-4 for reference purposes and its
 performance may not have been optimized. This software module is an implementation
 of one or more tools as specified by the ISO/IEC 23003-4 standard. ISO/IEC gives
 you a royalty-free, worldwide, non-exclusive, copyright license to copy, distribute,
 and make derivative works of this software module or modifications  thereof for use
 in implementations or products claiming conformance to the ISO/IEC 23003-4 standard
 and which satisfy any specified conformance criteria. Those intending to use this
 software module in products are advised that its use may infringe existing patents.
 ISO/IEC have no liability for use of this software module or modifications thereof.
 Copyright is not released for products that do not conform to the ISO/IEC 23003-4
 standard.
 
 Apple Inc. and Fraunhofer IIS retains full right to modify and use the code for its
 own purpose, assign or donate the code to a third party and to inhibit third parties
 from using the code for products that do not conform to MPEG-related ITU Recommenda-
 tions and/or ISO/IEC International Standards.
 
 This copyright notice must be included in all copies or derivative works.
 
 Copyright (c) ISO/IEC 2015.
 
 ***********************************************************************************/

#include <stdio.h>
#include "uniDrc.h"
#include "uniDrcInterface.h"
#include "uniDrcSelectionProcess_api.h"
#include "uniDrcHostParams.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */
    
/*****************************************************************/
/* systemInterface() parameters */
/*****************************************************************/
    
#define MAX_NUM_DOWNMIX_ID_REQUESTS_LOCAL 3
typedef struct {
    int   targetConfigRequestType;
    int   numDownmixIdRequests;
    int   downmixIdRequested[MAX_NUM_DOWNMIX_ID_REQUESTS_LOCAL];
    int   targetLayoutRequested;
    int   targetChannelCountRequested;
} SystemInterface_local;

const SystemInterface_local systemInterface_local[] = {
#if !MPEG_H_SYNTAX
    
    /* 01 */ {0, 1, {0,0,0}, 0, 0},
    /* 02 */ {0, 1, {2,0,0}, 0, 0},
#if ENABLE_ADDITIONAL_TESTS
    /* 03 */ {0, 1, {0,0,0}, 0, 0},
    /* 04 */ {0, 1, {0,0,0}, 0, 0},
    /* 05 */ {0, 1, {0,0,0}, 0, 0},
    /* 06 */ {0, 1, {0,0,0}, 0, 0},
    /* 07 */ {0, 1, {0,0,0}, 0, 0},
    /* 08 */ {0, 1, {0,0,0}, 0, 0},
    /* 09 */ {0, 1, {0,0,0}, 0, 0},
    /* 10 */ {0, 1, {0,0,0}, 0, 0},
    /* 11 */ {0, 1, {0,0,0}, 0, 0},
    /* 12 */ {0, 1, {0,0,0}, 0, 0},
    /* 13 */ {0, 1, {0,0,0}, 0, 0},
    /* 14 */ {0, 1, {0,0,0}, 0, 0},
    /* 15 */ {0, 1, {0,0,0}, 0, 0},
    /* 16 */ {0, 1, {0,0,0}, 0, 0},
    /* 17 */ {0, 1, {0,0,0}, 0, 0},
    /* 18 */ {0, 1, {0,0,0}, 0, 0},
    /* 19 */ {0, 1, {0,0,0}, 0, 0},
    /* 20 */ {0, 1, {2,0,0}, 0, 0},
    /* 21 */ {0, 1, {2,0,0}, 0, 0},
    /* 22 */ {0, 1, {2,0,0}, 0, 0},
    
    /* 23 */ {0, 1, {0,0,0}, 0, 0},
    /* 24 */ {0, 1, {0,0,0}, 0, 0},
    /* 25 */ {0, 1, {1,0,0}, 0, 0},
    /* 26 */ {0, 1, {1,0,0}, 0, 0},
    /* 27 */ {1, 1, {0,0,0}, 4, 0},
    /* 28 */ {2, 1, {0,0,0}, 0, 5},
    /* 29 */ {2, 1, {0,0,0}, 0, 2},
    /* 30 */ {0, 1, {1,0,0}, 0, 2},
    /* 31 */ {0, 1, {0,0,0}, 0, 2},
    /* 32 */ {0, 1, {0,0,0}, 0, 0},
    /* 33 */ {0, 1, {4,0,0}, 0, 0},
    /* 34 */ {0, 1, {5,0,0}, 0, 0},
    /* DRC off */
    /* 35 */ {0, 1, {0,0,0}, 0, 0},
    /* 36 */ {0, 1, {0,0,0}, 0, 0},
/* AMD1 */
    /* 37 */ {0, 1, {0,0,0}, 0, 0},
    /* 38 */ {0, 1, {0,0,0}, 0, 0},
    /* 39 */ {0, 1, {0,0,0}, 0, 0},
#endif
    
#else /* MPEG-H */
    
    /* 01 */ {0, 1, {0,0,0}, 0, 0},
    /* 02 */ {0, 1, {0,0,0}, 0, 0},
    /* 03 */ {0, 1, {0,0,0}, 0, 0},
    /* 04 */ {0, 1, {0,0,0}, 0, 0},
    /* 05 */ {0, 1, {0,0,0}, 0, 0},
    /* 06 */ {0, 1, {0,0,0}, 0, 0},
    
    /* 07 */ {0, 1, {0,0,0}, 0, 0},
    /* 08 */ {0, 1, {1,0,0}, 0, 0},
    /* 09 */ {0, 1, {0,0,0}, 0, 0},
    /* 10 */ {0, 1, {0,0,0}, 0, 0},
    
    /* 11 */ {0, 1, {1,0,0}, 0, 0},
    /* 12 */ {0, 1, {1,0,0}, 0, 0},
    /* 13 */ {0, 1, {1,0,0}, 0, 0},
    /* 14 */ {0, 1, {2,0,0}, 0, 0},
    /* 15 */ {0, 1, {0,0,0}, 0, 0},
    /* 16 */ {0, 1, {3,0,0}, 0, 0},
    /* 17 */ {0, 1, {1,0,0}, 0, 0},
    /* 18 */ {0, 1, {0,0,0}, 0, 0},
    
    /* 19 */ {0, 1, {2,0,0}, 0, 0},
    /* 20 */ {0, 1, {0,0,0}, 0, 0}
    
#endif
};

/*****************************************************************/
/* loudnessNormalizationControlInterface() parameters */
/*****************************************************************/
    
typedef struct {
    int   loudnessNormalizationOn;
    float targetLoudness;
} LoudnessNormalizationControlInterface_local;

const LoudnessNormalizationControlInterface_local loudnessNormalizationControlInterface_local[] = {
#if !MPEG_H_SYNTAX
    
    /* 01 */ {0,0.0f},
    /* 02 */ {0,0.0f},
#if ENABLE_ADDITIONAL_TESTS
    /* 03 */ {1,-32.0f},
    /* 04 */ {1,-24.0f},
    /* 05 */ {1,-24.0f},
    /* 06 */ {0,0.0f},
    /* 07 */ {1,-24.0f},
    /* 08 */ {0,0.0f},
    /* 09 */ {0,0.0f},
    /* 10 */ {1,-36.0f},
    /* 11 */ {1,-8.0f},
    /* 12 */ {1,-8.0f},
    /* 13 */ {1,-8.0f},
    /* 14 */ {1,-36.0f},
    /* 15 */ {1,-36.0f},
    /* 16 */ {1,-12.0f},
    /* 17 */ {1,-12.0f},
    /* 18 */ {1,-12.0f},
    /* 19 */ {1,-20.0f},
    /* 20 */ {1,-8.0f},
    /* 21 */ {1,-15.0f},
    /* 22 */ {1,-20.0f},
    
    /* 23 */ {1,-23.0f},
    /* 24 */ {1,-27.0f},
    /* 25 */ {1,-17.0f},
    /* 26 */ {1,-17.0f},
    /* 27 */ {1,-17.0f},
    /* 28 */ {1,-25.0f},
    /* 29 */ {1,-25.0f},
    /* 30 */ {1,-5.0f},
    /* 31 */ {1,-5.0f},
    /* 32 */ {0,0.0f},
    /* 33 */ {0,0.0f},
    /* 34 */ {0,0.0f},
    /* DRC off */
    /* 35 */ {1,-30.0f},
    /* 36 */ {1,-16.0f},
/* AMD1 */
    /* 37 */ {0,0.0f},
    /* 38 */ {0,0.0f},
    /* 39 */ {1,-10.0f},
#endif
    
#else /* MPEG-H */
    
    /* 01 */ {1,-23.0f},
    /* 02 */ {1,-23.0f},
    /* 03 */ {1,-23.0f},
    /* 04 */ {1,-23.0f},
    /* 05 */ {1,-20.0f},
    /* 06 */ {1,-23.0f},
    
    /* 07 */ {1,-31.0f},
    /* 08 */ {1,-31.0f},
    /* 09 */ {1,-24.0f},
    /* 10 */ {1,-16.0f},
    
    /* 11 */ {1,-30.0f},
    /* 12 */ {1,-30.0f},
    /* 13 */ {1,-30.0f},
    /* 14 */ {1,-30.0f},
    /* 15 */ {1,-30.0f},
    /* 16 */ {1,-10.0f},
    /* 17 */ {1,-30.0f},
    /* 18 */ {1,-30.0f},
    
    /* 19 */ {1,-24.0f},
    /* 20 */ {1,-24.0f}
    
#endif
};
    
/*****************************************************************/
/* loudnessNormalizationParameterInterface() parameters */
/*****************************************************************/
    
typedef struct {
    int   albumMode;
    int   peakLimiterPresent;
    int   loudnessDeviationMax;
    int   loudnessMeasurementMethod;
    int   loudnessMeasurementSystem;
    int   loudnessMeasurementPreProc;
    int   deviceCutOffFrequency;
    float loudnessNormalizationGainDbMax;
    float loudnessNormalizationGainModificationDb;
    float outputPeakLevelMax;
} LoudnessNormalizationParameterInterface_local;

const LoudnessNormalizationParameterInterface_local loudnessNormalizationParameterInterface_local[] = {
#if !MPEG_H_SYNTAX
    
    /* 01 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 0.0f},
    /* 02 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 0.0f},
#if ENABLE_ADDITIONAL_TESTS
    /* 03 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 04 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 05 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 06 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 07 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 08 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 09 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 4.0f},
    /* 10 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 4.0f},
    /* 11 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 0.0f},
    /* 12 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 13 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 14 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 15 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 16 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_EXPERT_PANEL, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 17 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 18 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 19 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 20 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 21 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 22 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    
    /* 23 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 24 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 25 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 26 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 27 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 28 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 29 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 30 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 31 */ {0, 0,                           18.f, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 32 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 33 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 34 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* DRC off */
    /* 35 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 36 */ {0, 1, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20,                                   10.0f, 0.0f, 1.0f},
/* AMD1 */
    /* 37 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_4, USER_LOUDNESS_PREPROCESSING_OFF, 20,                                   10.0f, 0.0f, 1.0f},
    /* 38 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_4, USER_LOUDNESS_PREPROCESSING_OFF, 20,                                   10.0f, 0.0f, 1.0f},
    /* 39 */ {0, 0,                              0, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS,    USER_MEASUREMENT_SYSTEM_BS_1770_4, USER_LOUDNESS_PREPROCESSING_OFF, 20,                                   12.0f, 0.0f, 1.0f},
#endif
    
#else /* MPEG-H */
    
    /* 01 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 02 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 03 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 04 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 05 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 06 */ {0, 0, LOUDNESS_DEVIATION_MAX_DEFAULT, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    
    /* 07 */ {0, 1, 0, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 0.0f},
    /* 08 */ {0, 1, 0, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 0.0f},
    /* 09 */ {0, 1, 0, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 0.0f},
    /* 10 */ {0, 1, 0, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 0.0f},
    
    /* 11 */ {0, 0, 0, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 12 */ {0, 0, 0, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 13 */ {0, 0, 0, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 14 */ {0, 0, 0, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 15 */ {0, 0, 0, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 16 */ {0, 0, 0, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 17 */ {0, 0, 0, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 18 */ {0, 0, 0, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    
    /* 19 */ {0, 0, 0, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f},
    /* 20 */ {0, 0, 0, USER_METHOD_DEFINITION_PROGRAM_LOUDNESS, USER_MEASUREMENT_SYSTEM_BS_1770_3, USER_LOUDNESS_PREPROCESSING_OFF, 20, LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT, 0.0f, 1.0f}

    
#endif
};

/*****************************************************************/
/* dynamicRangeControlInterface() parameters */
/*****************************************************************/
    
#define MAX_NUM_DRC_FEATURE_REQUESTS_LOCAL 3
typedef struct {
    int   dynamicRangeControlOn;
    int   numDrcFeatureRequests;
    int   drcFeatureRequestType[MAX_NUM_DRC_FEATURE_REQUESTS_LOCAL];
    int   dynRangeMeasurementRequestType;
    int   dynRangeRequestedIsRange;
    float dynamicRangeRequestValue;
    float dynamicRangeRequestValueMin;
    float dynamicRangeRequestValueMax;
    int   drcCharacteristicRequest;
} DynamicRangeControlInterface_local;

const DynamicRangeControlInterface_local dynamicRangeControlInterface_local[] = {
#if !MPEG_H_SYNTAX
    
    /* 01 */ {1, 3, {MATCH_EFFECT_TYPE, MATCH_DYNAMIC_RANGE, MATCH_DRC_CHARACTERISTIC}, SHORT_TERM_LOUDNESS_TO_AVG,   TRUE,   5.0f, 3.0f, 10.0f, 3},
    /* 02 */ {1, 3, {MATCH_EFFECT_TYPE, MATCH_DYNAMIC_RANGE, MATCH_DRC_CHARACTERISTIC}, TOP_OF_LOUDNESS_RANGE_TO_AVG, FALSE,  5.0f, 3.0f, 10.0f, 3},
#if ENABLE_ADDITIONAL_TESTS
    /* 03 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DYNAMIC_RANGE, MATCH_DRC_CHARACTERISTIC}, TOP_OF_LOUDNESS_RANGE_TO_AVG, TRUE,   5.0f, 3.0f, 10.0f, 3},
    /* 04 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DYNAMIC_RANGE, MATCH_DRC_CHARACTERISTIC}, TOP_OF_LOUDNESS_RANGE_TO_AVG, TRUE,   5.0f, 3.0f, 10.0f, 3},
    /* 05 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DYNAMIC_RANGE, MATCH_DRC_CHARACTERISTIC}, TOP_OF_LOUDNESS_RANGE_TO_AVG, TRUE,   5.0f, 3.0f, 10.0f, 3},
    /* 06 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DYNAMIC_RANGE, MATCH_DRC_CHARACTERISTIC}, TOP_OF_LOUDNESS_RANGE_TO_AVG, TRUE,   5.0f, 3.0f, 10.0f, 3},
    /* 07 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DYNAMIC_RANGE, MATCH_DRC_CHARACTERISTIC}, TOP_OF_LOUDNESS_RANGE_TO_AVG, TRUE,   5.0f, 3.0f, 10.0f, 3},
    /* 08 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DYNAMIC_RANGE, MATCH_DRC_CHARACTERISTIC}, TOP_OF_LOUDNESS_RANGE_TO_AVG, TRUE,   5.0f, 3.0f, 10.0f, 3},
    /* 09 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DYNAMIC_RANGE, MATCH_DRC_CHARACTERISTIC}, TOP_OF_LOUDNESS_RANGE_TO_AVG, TRUE,   5.0f, 3.0f, 10.0f, 3},
    /* 10 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DYNAMIC_RANGE, MATCH_DRC_CHARACTERISTIC}, TOP_OF_LOUDNESS_RANGE_TO_AVG, TRUE,   5.0f, 3.0f, 10.0f, 3},
    /* 11 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DYNAMIC_RANGE, MATCH_DRC_CHARACTERISTIC}, TOP_OF_LOUDNESS_RANGE_TO_AVG, TRUE,   5.0f, 3.0f, 10.0f, 3},
    /* 12 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DYNAMIC_RANGE, MATCH_DRC_CHARACTERISTIC}, TOP_OF_LOUDNESS_RANGE_TO_AVG, TRUE,   5.0f, 3.0f, 10.0f, 3},
    /* 13 */ {1, 1, {MATCH_DYNAMIC_RANGE, MATCH_DRC_CHARACTERISTIC, MATCH_EFFECT_TYPE}, TOP_OF_LOUDNESS_RANGE_TO_AVG, TRUE,   5.0f, 3.0f, 10.0f, 3},
    /* 14 */ {1, 1, {MATCH_DYNAMIC_RANGE, MATCH_DRC_CHARACTERISTIC, MATCH_EFFECT_TYPE}, SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 15 */ {1, 1, {MATCH_DYNAMIC_RANGE, MATCH_DRC_CHARACTERISTIC, MATCH_EFFECT_TYPE}, TOP_OF_LOUDNESS_RANGE_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 16 */ {1, 1, {MATCH_DYNAMIC_RANGE, MATCH_DRC_CHARACTERISTIC, MATCH_EFFECT_TYPE}, TOP_OF_LOUDNESS_RANGE_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 17 */ {1, 1, {MATCH_DRC_CHARACTERISTIC, MATCH_EFFECT_TYPE, MATCH_DYNAMIC_RANGE}, TOP_OF_LOUDNESS_RANGE_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 18 */ {1, 2, {MATCH_DRC_CHARACTERISTIC, MATCH_EFFECT_TYPE, MATCH_DYNAMIC_RANGE}, TOP_OF_LOUDNESS_RANGE_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 1},
    /* 19 */ {1, 3, {MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE, MATCH_EFFECT_TYPE}, SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 20 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 21 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 22 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    
    /* 23 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 24 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 25 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 26 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 27 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 28 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 29 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 30 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 31 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 32 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 33 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 34 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* DRC off */
    /* 35 */ {0, 0, {MATCH_EFFECT_TYPE, MATCH_EFFECT_TYPE, MATCH_EFFECT_TYPE},          SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 36 */ {0, 0, {MATCH_EFFECT_TYPE, MATCH_EFFECT_TYPE, MATCH_EFFECT_TYPE},          SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
/* AMD1 */
    /* 37 */ {0, 0, {MATCH_EFFECT_TYPE, MATCH_EFFECT_TYPE, MATCH_EFFECT_TYPE},          SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 38 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_EFFECT_TYPE, MATCH_EFFECT_TYPE},          SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 39 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_EFFECT_TYPE, MATCH_EFFECT_TYPE},          SHORT_TERM_LOUDNESS_TO_AVG,   FALSE, 25.0f, 3.0f, 10.0f, 3},
#endif
    
#else /* MPEG-H */
    
    /* 01 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 02 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 03 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 04 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 05 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 06 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},

    /* 07 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 08 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 09 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 10 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    
    /* 11 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 12 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 13 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 14 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 15 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 16 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 17 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 18 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    
    /* 19 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3},
    /* 20 */ {1, 1, {MATCH_EFFECT_TYPE, MATCH_DRC_CHARACTERISTIC, MATCH_DYNAMIC_RANGE}, SHORT_TERM_LOUDNESS_TO_AVG, FALSE, 25.0f, 3.0f, 10.0f, 3}

#endif
};

/*****************************************************************/
/* drcEffectTypeRequest() parameters */
/*****************************************************************/
    
#define MAX_NUM_DRC_EFFECT_TYPE_REQUESTS_LOCAL 5
typedef struct {
    int numDrcEffectTypeRequests;
    int numDrcEffectTypeRequestsDesired;
    int drcEffectTypeRequest[MAX_NUM_DRC_EFFECT_TYPE_REQUESTS_LOCAL];
} DrcEffectTypeRequest_local;

const DrcEffectTypeRequest_local drcEffectTypeRequest_local[] = {
#if !MPEG_H_SYNTAX
        /* 01 */ {1, 1, {EFFECT_TYPE_REQUESTED_NIGHT,  EFFECT_TYPE_REQUESTED_GENERAL_COMPR, EFFECT_TYPE_REQUESTED_ARTISTIC, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 02 */ {4, 1, {EFFECT_TYPE_REQUESTED_LOWLEVEL,  EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_ARTISTIC, EFFECT_TYPE_REQUESTED_DIALOG}},
#if ENABLE_ADDITIONAL_TESTS
        /* 03 */ {4, 1, {EFFECT_TYPE_REQUESTED_NONE,  EFFECT_TYPE_REQUESTED_GENERAL_COMPR, EFFECT_TYPE_REQUESTED_ARTISTIC, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 04 */ {4, 1, {EFFECT_TYPE_REQUESTED_NIGHT,  EFFECT_TYPE_REQUESTED_GENERAL_COMPR, EFFECT_TYPE_REQUESTED_ARTISTIC, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 05 */ {4, 1, {EFFECT_TYPE_REQUESTED_NOISY,  EFFECT_TYPE_REQUESTED_GENERAL_COMPR, EFFECT_TYPE_REQUESTED_ARTISTIC, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 06 */ {4, 1, {EFFECT_TYPE_REQUESTED_DIALOG,  EFFECT_TYPE_REQUESTED_GENERAL_COMPR, EFFECT_TYPE_REQUESTED_ARTISTIC, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_LOWLEVEL}},
        /* 07 */ {4, 1, {EFFECT_TYPE_REQUESTED_GENERAL_COMPR, EFFECT_TYPE_REQUESTED_ARTISTIC, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 08 */ {4, 2, {EFFECT_TYPE_REQUESTED_GENERAL_COMPR, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 09 */ {4, 2, {EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 10 */ {4, 2, {EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 11 */ {4, 1, {EFFECT_TYPE_REQUESTED_GENERAL_COMPR, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 12 */ {4, 1, {EFFECT_TYPE_REQUESTED_NONE, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 13 */ {4, 1, {EFFECT_TYPE_REQUESTED_NONE, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 14 */ {4, 1, {EFFECT_TYPE_REQUESTED_NONE, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 15 */ {4, 1, {EFFECT_TYPE_REQUESTED_NONE, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 16 */ {4, 1, {EFFECT_TYPE_REQUESTED_NONE, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 17 */ {4, 1, {EFFECT_TYPE_REQUESTED_NONE, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 18 */ {4, 1, {EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 19 */ {4, 1, {EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 20 */ {4, 1, {EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 21 */ {4, 1, {EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 22 */ {4, 1, {EFFECT_TYPE_REQUESTED_DIALOG, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_DIALOG}},
        
        /* 23 */ {4, 1, {EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 24 */ {4, 1, {EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 25 */ {4, 1, {EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 26 */ {4, 1, {EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 27 */ {4, 1, {EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 28 */ {4, 1, {EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 29 */ {4, 2, {EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 30 */ {4, 2, {EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 31 */ {4, 2, {EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 32 */ {4, 1, {EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 33 */ {4, 1, {EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 34 */ {1, 1, {EFFECT_TYPE_REQUESTED_NONE, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* DRC off */
        /* 35 */ {0, 0, {EFFECT_TYPE_REQUESTED_NONE, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 36 */ {0, 0, {EFFECT_TYPE_REQUESTED_NONE, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
/* AMD1 */
        /* 37 */ {0, 0, {EFFECT_TYPE_REQUESTED_NONE, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 38 */ {1, 1, {EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 39 */ {1, 1, {EFFECT_TYPE_REQUESTED_GENERAL_COMPR, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
#endif
        
#else /* MPEG-H */
        
        /* 01 */ {4, 1, {EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 02 */ {4, 1, {EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 03 */ {4, 1, {EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 04 */ {4, 1, {EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 05 */ {4, 1, {EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 06 */ {4, 1, {EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
    
        /* 07 */ {1, 1, {EFFECT_TYPE_REQUESTED_NONE, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 08 */ {1, 1, {EFFECT_TYPE_REQUESTED_NONE, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 09 */ {1, 1, {EFFECT_TYPE_REQUESTED_GENERAL_COMPR, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 10 */ {1, 1, {EFFECT_TYPE_REQUESTED_GENERAL_COMPR, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
    
        /* 11 */ {4, 1, {EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 12 */ {4, 1, {EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 13 */ {4, 1, {EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 14 */ {4, 1, {EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 15 */ {4, 1, {EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 16 */ {4, 1, {EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 17 */ {4, 1, {EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 18 */ {4, 1, {EFFECT_TYPE_REQUESTED_NIGHT, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
    
        /* 19 */ {4, 1, {EFFECT_TYPE_REQUESTED_GENERAL_COMPR, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}},
        /* 20 */ {4, 1, {EFFECT_TYPE_REQUESTED_GENERAL_COMPR, EFFECT_TYPE_REQUESTED_NOISY, EFFECT_TYPE_REQUESTED_LOWLEVEL, EFFECT_TYPE_REQUESTED_LIMITED, EFFECT_TYPE_REQUESTED_DIALOG}}

#endif
};
    
/*****************************************************************/
/* dynamicRangeControlParameterInterface() parameters */
/*****************************************************************/
    
typedef struct {
    float compress;
    float boost;
    int   drcCharacteristicTarget;
} DynamicRangeControlParameterInterface_local;

const DynamicRangeControlParameterInterface_local dynamicRangeControlParameterInterface_local[] = {
#if !MPEG_H_SYNTAX
    
    /* 01 */ {1.0f,1.0f,0},
    /* 02 */ {1.0f,1.0f,0},
#if ENABLE_ADDITIONAL_TESTS
    /* 03 */ {1.0f,1.0f,0},
    /* 04 */ {1.0f,1.0f,0},
    /* 05 */ {1.0f,1.0f,0},
    /* 06 */ {1.0f,1.0f,0},
    /* 07 */ {1.0f,1.0f,0},
    /* 08 */ {1.0f,1.0f,0},
    /* 09 */ {1.0f,1.0f,0},
    /* 10 */ {1.0f,1.0f,0},
    /* 11 */ {1.0f,1.0f,0},
    /* 12 */ {1.0f,1.0f,0},
    /* 13 */ {1.0f,1.0f,0},
    /* 14 */ {1.0f,1.0f,0},
    /* 15 */ {1.0f,1.0f,0},
    /* 16 */ {1.0f,1.0f,0},
    /* 17 */ {1.0f,1.0f,0},
    /* 18 */ {1.0f,1.0f,0},
    /* 19 */ {1.0f,1.0f,0},
    /* 20 */ {1.0f,1.0f,0},
    /* 21 */ {1.0f,1.0f,0},
    /* 22 */ {1.0f,1.0f,0},
    /* 23 */ {1.0f,1.0f,0},
    /* 24 */ {1.0f,1.0f,0},
    /* 25 */ {1.0f,1.0f,0},
    /* 26 */ {1.0f,1.0f,0},
    /* 27 */ {1.0f,1.0f,0},
    /* 28 */ {1.0f,1.0f,0},
    /* 29 */ {1.0f,1.0f,0},
    /* 30 */ {1.0f,1.0f,0},
    /* 31 */ {1.0f,1.0f,0},
    /* 32 */ {1.0f,1.0f,0},
    /* 33 */ {1.0f,1.0f,0},
    /* 34 */ {1.0f,1.0f,0},
    /* DRC off */
    /* 35 */ {1.0f,1.0f,0},
    /* 36 */ {1.0f,1.0f,0},
/* AMD1 */
    /* 37 */ {1.0f,1.0f,0},
    /* 38 */ {1.0f,1.0f,0},
    /* 39 */ {1.0f,1.0f,0},
#endif
    
#else /* MPEG-H */
    
    /* 01 */ {1.0f,1.0f,0},
    /* 02 */ {1.0f,1.0f,0},
    /* 03 */ {1.0f,1.0f,0},
    /* 04 */ {1.0f,1.0f,0},
    /* 05 */ {1.0f,1.0f,0},
    /* 06 */ {1.0f,1.0f,0},
    
    /* 07 */ {1.0f,1.0f,0},
    /* 08 */ {1.0f,1.0f,0},
    /* 09 */ {1.0f,1.0f,0},
    /* 10 */ {1.0f,1.0f,0},
    
    /* 11 */ {1.0f,1.0f,0},
    /* 12 */ {1.0f,1.0f,0},
    /* 13 */ {1.0f,1.0f,0},
    /* 14 */ {1.0f,1.0f,0},
    /* 15 */ {1.0f,1.0f,0},
    /* 16 */ {1.0f,1.0f,0},
    /* 17 */ {1.0f,1.0f,0},
    /* 18 */ {1.0f,1.0f,0},
    
    /* 19 */ {1.0f,1.0f,0},
    /* 20 */ {1.0f,1.0f,0}
    
#endif
};
    
#if MPEG_H_SYNTAX
/*****************************************************************/
/* dynamicRangeControlParameterInterface() parameters */
/*****************************************************************/
    
typedef struct {
    int numGroupIdsRequested;
    int groupIdRequested[3];
    int numGroupPresetIdsRequested;
    int groupPresetIdRequested[3];
    int numMembersGroupPresetIdsRequested[3];
    int groupPresetIdRequestedPreference;
} MpeghParameterInterface_local ;
    
const MpeghParameterInterface_local mpeghParameterInterface_local[] = {
    
    /* 01 */ {0, {0,0,0}, 0, {0,0,0}, {1,1,1}, -1},
    /* 02 */ {2, {1,2,0}, 0, {0,0,0}, {1,1,1}, -1},
    /* 03 */ {2, {1,2,0}, 0, {0,0,0}, {1,1,1}, -1},
    /* 04 */ {2, {1,2,0}, 2, {1,2,0}, {1,2,1}, -1},
    /* 05 */ {2, {1,2,0}, 2, {1,2,0}, {1,2,1}, -1},
    /* 06 */ {0, {0,0,0}, 2, {1,2,0}, {1,2,1},  1},
    
    /* 07 */ {0, {0,0,0}, 0, {0,0,0}, {1,1,1}, -1},
    /* 08 */ {0, {0,0,0}, 0, {0,0,0}, {1,1,1}, -1},
    /* 09 */ {0, {0,0,0}, 0, {0,0,0}, {1,1,1}, -1},
    /* 10 */ {0, {0,0,0}, 0, {0,0,0}, {1,1,1}, -1},
    
    /* 11 */ {0, {0,0,0}, 1, {2,8,0}, {1,1,1}, -1},
    /* 12 */ {0, {0,0,0}, 1, {2,4,0}, {1,1,1}, -1},
    /* 13 */ {2, {10,7,0}, 2, {2,3,0}, {1,1,1}, -1},
    /* 14 */ {2, {10,2,0}, 2, {1,5,0}, {1,2,1}, -1},
    /* 15 */ {2, {1,2,0}, 2, {1,2,0}, {1,2,1}, -1},
    /* 16 */ {2, {1,3,0}, 2, {1,3,0}, {1,2,1},  1},
    /* 17 */ {2, {1,7,0}, 2, {1,3,0}, {1,2,1},  1},
    /* 18 */ {2, {1,5,0}, 2, {5,3,0}, {1,2,1},  1},
    
    /* 19 */ {0, {0,0,0}, 0, {0,0,0}, {0,0,0},  -1},
    /* 20 */ {0, {0,0,0}, 1, {1,0,0}, {1,0,0},  1}
    
};
#endif

/*****************************************************************/
/* LoudnessEqParameterInterface() parameters */
/*****************************************************************/
    
#if MPEG_D_DRC_EXTENSION_V1
typedef struct {
    int loudnessEqRequest;
    float sensitivity;
    float playbackGain;
} LoudnessEqParameterInterface_local;

const LoudnessEqParameterInterface_local loudnessEqParameterInterface_local[] = {
#if !MPEG_H_SYNTAX
    
    /* 01 */ {0, 90.0f, -15.0f},
    /* 02 */ {0, 90.0f, -15.0f},
#if ENABLE_ADDITIONAL_TESTS
    /* 03 */ {0, 90.0f, -15.0f},
    /* 04 */ {0, 90.0f, -15.0f},
    /* 05 */ {0, 90.0f, -15.0f},
    /* 06 */ {0, 90.0f, -15.0f},
    /* 07 */ {0, 90.0f, -15.0f},
    /* 08 */ {0, 90.0f, -15.0f},
    /* 09 */ {0, 90.0f, -15.0f},
    /* 10 */ {0, 90.0f, -15.0f},
    /* 11 */ {0, 90.0f, -15.0f},
    /* 12 */ {0, 90.0f, -15.0f},
    /* 13 */ {0, 90.0f, -15.0f},
    /* 14 */ {0, 90.0f, -15.0f},
    /* 15 */ {0, 90.0f, -15.0f},
    /* 16 */ {0, 90.0f, -15.0f},
    /* 17 */ {0, 90.0f, -15.0f},
    /* 18 */ {0, 90.0f, -15.0f},
    /* 19 */ {0, 90.0f, -15.0f},
    /* 20 */ {0, 90.0f, -15.0f},
    /* 21 */ {0, 90.0f, -15.0f},
    /* 22 */ {0, 90.0f, -15.0f},
    /* 23 */ {0, 90.0f, -15.0f},
    /* 24 */ {0, 90.0f, -15.0f},
    /* 25 */ {0, 90.0f, -15.0f},
    /* 26 */ {0, 90.0f, -15.0f},
    /* 27 */ {0, 90.0f, -15.0f},
    /* 28 */ {0, 90.0f, -15.0f},
    /* 29 */ {0, 90.0f, -15.0f},
    /* 30 */ {0, 90.0f, -15.0f},
    /* 31 */ {0, 90.0f, -15.0f},
    /* 32 */ {0, 90.0f, -15.0f},
    /* 33 */ {0, 90.0f, -15.0f},
    /* 34 */ {0, 90.0f, -15.0f},
    /* DRC off */
    /* 35 */ {0, 90.0f, -15.0f},
    /* 36 */ {0, 90.0f, -15.0f},
/* AMD1 */
    /* 37 */ {2, 90.0f, -15.0f},
    /* 38 */ {0, 90.0f, -15.0f},
    /* 39 */ {0, 90.0f, -15.0f},
#endif
    
#else /* MPEG-H */
    
    /* 01 */ {0, 90.0f, -15.0f},
    /* 02 */ {0, 90.0f, -15.0f},
    /* 03 */ {0, 90.0f, -15.0f},
    /* 04 */ {0, 90.0f, -15.0f},
    /* 05 */ {0, 90.0f, -15.0f},
    /* 06 */ {0, 90.0f, -15.0f},
    
    /* 07 */ {0, 90.0f, -15.0f},
    /* 08 */ {0, 90.0f, -15.0f},
    /* 09 */ {0, 90.0f, -15.0f},
    /* 10 */ {0, 90.0f, -15.0f},
    
    /* 11 */ {0, 90.0f, -15.0f},
    /* 12 */ {0, 90.0f, -15.0f},
    /* 13 */ {0, 90.0f, -15.0f},
    /* 14 */ {0, 90.0f, -15.0f},
    /* 15 */ {0, 90.0f, -15.0f},
    /* 16 */ {0, 90.0f, -15.0f},
    /* 17 */ {0, 90.0f, -15.0f},
    /* 18 */ {0, 90.0f, -15.0f},
    
    /* 19 */ {0, 90.0f, -15.0f},
    /* 20 */ {0, 90.0f, -15.0f}
    
#endif
    };
    
/*****************************************************************/
/* EqualizationControlInterface() parameters */
/*****************************************************************/
    
typedef struct {
    int eqSetPurposeRequest;
} EqualizationControlInterface_local;

const EqualizationControlInterface_local equalizationControlInterface_local[] = {
#if !MPEG_H_SYNTAX
    
    /* 01 */ {0},
    /* 02 */ {0},
#if ENABLE_ADDITIONAL_TESTS
    /* 03 */ {0},
    /* 04 */ {0},
    /* 05 */ {0},
    /* 06 */ {0},
    /* 07 */ {0},
    /* 08 */ {0},
    /* 09 */ {0},
    /* 10 */ {0},
    /* 11 */ {0},
    /* 12 */ {0},
    /* 13 */ {0},
    /* 14 */ {0},
    /* 15 */ {0},
    /* 16 */ {0},
    /* 17 */ {0},
    /* 18 */ {0},
    /* 19 */ {0},
    /* 20 */ {0},
    /* 21 */ {0},
    /* 22 */ {0},
    /* 23 */ {0},
    /* 24 */ {0},
    /* 25 */ {0},
    /* 26 */ {0},
    /* 27 */ {0},
    /* 28 */ {0},
    /* 29 */ {0},
    /* 30 */ {0},
    /* 31 */ {0},
    /* 32 */ {0},
    /* 33 */ {0},
    /* 34 */ {0},
    /* DRC off */
    /* 35 */ {0},
    /* 36 */ {0},
/* AMD1 */
    /* 37 */ {1},
    /* 38 */ {0},
    /* 39 */ {0},
#endif
    
#else /* MPEG-H */
    
    /* 01 */ {0},
    /* 02 */ {0},
    /* 03 */ {0},
    /* 04 */ {0},
    /* 05 */ {0},
    /* 06 */ {0},
    
    /* 07 */ {0},
    /* 08 */ {0},
    /* 09 */ {0},
    /* 10 */ {0},
    
    /* 11 */ {0},
    /* 12 */ {0},
    /* 13 */ {0},
    /* 14 */ {0},
    /* 15 */ {0},
    /* 16 */ {0},
    /* 17 */ {0},
    /* 18 */ {0},
    
    /* 19 */ {0},
    /* 20 */ {0}
    
#endif
};

#endif
    
int
setDefaultParams_selectionProcess(UniDrcSelProcParams* pUniDrcSelProcParams)
{
    /* system parameters */
    pUniDrcSelProcParams->baseChannelCount = -1;
    pUniDrcSelProcParams->baseLayout = -1;
    pUniDrcSelProcParams->targetConfigRequestType = 0;
    pUniDrcSelProcParams->numDownmixIdRequests = 0;
    
    /* loudness normalization parameters */
    pUniDrcSelProcParams->albumMode = FALSE;
#if !MPEG_H_SYNTAX
    pUniDrcSelProcParams->peakLimiterPresent = FALSE;
#else
    pUniDrcSelProcParams->peakLimiterPresent = TRUE;
#endif
    pUniDrcSelProcParams->loudnessNormalizationOn = FALSE;
    pUniDrcSelProcParams->targetLoudness = -24.0f;
#if !MPEG_H_SYNTAX
    pUniDrcSelProcParams->loudnessDeviationMax = LOUDNESS_DEVIATION_MAX_DEFAULT;
#else
    pUniDrcSelProcParams->loudnessDeviationMax = 0;
#endif
    pUniDrcSelProcParams->loudnessMeasurementMethod = USER_METHOD_DEFINITION_DEFAULT;
    pUniDrcSelProcParams->loudnessMeasurementSystem = USER_MEASUREMENT_SYSTEM_DEFAULT;
    pUniDrcSelProcParams->loudnessMeasurementPreProc = USER_LOUDNESS_PREPROCESSING_DEFAULT;
    pUniDrcSelProcParams->deviceCutOffFrequency = 500;
    pUniDrcSelProcParams->loudnessNormalizationGainDbMax = LOUDNESS_NORMALIZATION_GAIN_MAX_DEFAULT; /* infinity as default */
    pUniDrcSelProcParams->loudnessNormalizationGainModificationDb = 0.0f;
    pUniDrcSelProcParams->outputPeakLevelMax = 0.0f;
    if (pUniDrcSelProcParams->peakLimiterPresent == TRUE) {
#if !MPEG_H_SYNTAX
        pUniDrcSelProcParams->outputPeakLevelMax = 6.0f;
#endif
    }
    
    /* dynamic range control parameters */
#ifdef AMD2_COR1
    pUniDrcSelProcParams->dynamicRangeControlOn = FALSE;
#else
    pUniDrcSelProcParams->dynamicRangeControlOn = TRUE;
#endif
    pUniDrcSelProcParams->numBandsSupported = 4; /* assuming time domain DRC */
    pUniDrcSelProcParams->numDrcFeatureRequests = 0;
    
    /*pUniDrcSelProcParams->drcFeatureRequestType[0] = MATCH_EFFECT_TYPE;
    pUniDrcSelProcParams->numDrcEffectTypeRequests[0] = 6;
    pUniDrcSelProcParams->numDrcEffectTypeRequestsDesired[0] = 1;
    pUniDrcSelProcParams->drcEffectTypeRequest[0][0] = EFFECT_TYPE_REQUESTED_NONE;
    pUniDrcSelProcParams->drcEffectTypeRequest[0][1] = EFFECT_TYPE_REQUESTED_GENERAL_COMPR;
    pUniDrcSelProcParams->drcEffectTypeRequest[0][2] = EFFECT_TYPE_REQUESTED_NIGHT;
    pUniDrcSelProcParams->drcEffectTypeRequest[0][3] = EFFECT_TYPE_REQUESTED_NOISY;
    pUniDrcSelProcParams->drcEffectTypeRequest[0][4] = EFFECT_TYPE_REQUESTED_LIMITED;
    pUniDrcSelProcParams->drcEffectTypeRequest[0][5] = EFFECT_TYPE_REQUESTED_LOWLEVEL;*/
    
    /* other parameters */
    pUniDrcSelProcParams->boost = 1.f;
    pUniDrcSelProcParams->compress = 1.f;
    pUniDrcSelProcParams->drcCharacteristicTarget = 0;
    
    /* MPEG-H parameters */
#if MPEG_H_SYNTAX
    pUniDrcSelProcParams->numGroupIdsRequested = 0;
    pUniDrcSelProcParams->numGroupPresetIdsRequested = 0;
#endif
#if MPEG_D_DRC_EXTENSION_V1
    pUniDrcSelProcParams->loudnessEqRequest = LOUD_EQ_REQUEST_OFF;
    pUniDrcSelProcParams->sensitivity = 90.0f;
    pUniDrcSelProcParams->playbackGain = -15.0f;
    pUniDrcSelProcParams->eqSetPurposeRequest = 0x0;
#endif
    return 0;
}

int
setCustomParams_selectionProcess(const int paramSetIndex,
                                 UniDrcSelProcParams* pUniDrcSelProcParams)
{
    int i, k;
    
    const SystemInterface_local* systemInterface = &(systemInterface_local[paramSetIndex-1]);
    
    const LoudnessNormalizationControlInterface_local* loudnessNormalizationControlInterface = &(loudnessNormalizationControlInterface_local[paramSetIndex-1]);
    const LoudnessNormalizationParameterInterface_local* loudnessNormalizationParameterInterface = &(loudnessNormalizationParameterInterface_local[paramSetIndex-1]);
    
    const DynamicRangeControlInterface_local* dynamicRangeControlInterface = &(dynamicRangeControlInterface_local[paramSetIndex-1]);
    const DrcEffectTypeRequest_local* drcEffectTypeRequest = &(drcEffectTypeRequest_local[paramSetIndex-1]);
    const DynamicRangeControlParameterInterface_local* dynamicRangeControlParameterInterface = &(dynamicRangeControlParameterInterface_local[paramSetIndex-1]);
    
#if MPEG_H_SYNTAX
    const MpeghParameterInterface_local* mpeghParameterInterface = &(mpeghParameterInterface_local[paramSetIndex-1]);
#endif
#if MPEG_D_DRC_EXTENSION_V1
    const LoudnessEqParameterInterface_local* loudnessEqParameterInterface = &(loudnessEqParameterInterface_local[paramSetIndex-1]);
    const EqualizationControlInterface_local* equalizationControlInterface = &(equalizationControlInterface_local[paramSetIndex-1]);
#endif
    /********************************************************************************/
    /* systemInterface */
    /********************************************************************************/


    pUniDrcSelProcParams->targetConfigRequestType = systemInterface->targetConfigRequestType;
    switch (systemInterface->targetConfigRequestType) {
        case 1:
            pUniDrcSelProcParams->targetLayoutRequested = systemInterface->targetLayoutRequested;
            break;
        case 2:
            pUniDrcSelProcParams->targetChannelCountRequested = systemInterface->targetChannelCountRequested;
            break;
        case 0:
        default:
            pUniDrcSelProcParams->numDownmixIdRequests = systemInterface->numDownmixIdRequests;
            for (i=0; i<systemInterface->numDownmixIdRequests; i++) {
                pUniDrcSelProcParams->downmixIdRequested[i] = systemInterface->downmixIdRequested[i];
            }
            break;
    }
    
    /********************************************************************************/
    /* loudnessNormalizationControlInterface */
    /********************************************************************************/
    
    pUniDrcSelProcParams->loudnessNormalizationOn = loudnessNormalizationControlInterface->loudnessNormalizationOn;
    pUniDrcSelProcParams->targetLoudness = loudnessNormalizationControlInterface->targetLoudness;
    
    /********************************************************************************/
    /* loudnessNormalizationParameterInterface */
    /********************************************************************************/
    
    pUniDrcSelProcParams->albumMode = loudnessNormalizationParameterInterface->albumMode;
    pUniDrcSelProcParams->peakLimiterPresent = loudnessNormalizationParameterInterface->peakLimiterPresent;
    pUniDrcSelProcParams->loudnessDeviationMax = loudnessNormalizationParameterInterface->loudnessDeviationMax;
    pUniDrcSelProcParams->loudnessMeasurementMethod = loudnessNormalizationParameterInterface->loudnessMeasurementMethod;
    pUniDrcSelProcParams->loudnessMeasurementSystem = loudnessNormalizationParameterInterface->loudnessMeasurementSystem;
    pUniDrcSelProcParams->loudnessMeasurementPreProc = loudnessNormalizationParameterInterface->loudnessMeasurementPreProc;
    pUniDrcSelProcParams->deviceCutOffFrequency= loudnessNormalizationParameterInterface->deviceCutOffFrequency;
    pUniDrcSelProcParams->loudnessNormalizationGainDbMax = loudnessNormalizationParameterInterface->loudnessNormalizationGainDbMax;
    pUniDrcSelProcParams->loudnessNormalizationGainModificationDb = loudnessNormalizationParameterInterface->loudnessNormalizationGainModificationDb;
    pUniDrcSelProcParams->outputPeakLevelMax = loudnessNormalizationParameterInterface->outputPeakLevelMax;

    /********************************************************************************/
    /* dynamicRangeControlInterface */
    /********************************************************************************/
    
    pUniDrcSelProcParams->dynamicRangeControlOn = dynamicRangeControlInterface->dynamicRangeControlOn;
    pUniDrcSelProcParams->numDrcFeatureRequests = dynamicRangeControlInterface->numDrcFeatureRequests;
    for (i=0; i<dynamicRangeControlInterface->numDrcFeatureRequests; i++)
    {
        pUniDrcSelProcParams->drcFeatureRequestType[i] = dynamicRangeControlInterface->drcFeatureRequestType[i];
        switch (dynamicRangeControlInterface->drcFeatureRequestType[i])
        {
            case MATCH_EFFECT_TYPE:
                pUniDrcSelProcParams->numDrcEffectTypeRequests[i] = drcEffectTypeRequest->numDrcEffectTypeRequests;
                pUniDrcSelProcParams->numDrcEffectTypeRequestsDesired[i] = drcEffectTypeRequest->numDrcEffectTypeRequestsDesired;
                for (k=0; k<drcEffectTypeRequest->numDrcEffectTypeRequests; k++)
                {
                    pUniDrcSelProcParams->drcEffectTypeRequest[i][k] = drcEffectTypeRequest->drcEffectTypeRequest[k];
                }
                break;
            case MATCH_DYNAMIC_RANGE:
                pUniDrcSelProcParams->dynamicRangeMeasurementRequestType[i] = dynamicRangeControlInterface->dynRangeMeasurementRequestType;
                pUniDrcSelProcParams->dynamicRangeRequestedIsRange[i] = dynamicRangeControlInterface->dynRangeRequestedIsRange;
                pUniDrcSelProcParams->dynamicRangeRequestValue[i] = dynamicRangeControlInterface->dynamicRangeRequestValue;
                pUniDrcSelProcParams->dynamicRangeRequestValueMin[i] = dynamicRangeControlInterface->dynamicRangeRequestValueMin;
                pUniDrcSelProcParams->dynamicRangeRequestValueMax[i] = dynamicRangeControlInterface->dynamicRangeRequestValueMax;
                break;
            case MATCH_DRC_CHARACTERISTIC:
                pUniDrcSelProcParams->drcCharacteristicRequest[i] = dynamicRangeControlInterface->drcCharacteristicRequest;
                break;
            default:
                fprintf(stderr, "ERROR: unknown feature requested (%d)\n", dynamicRangeControlInterface->drcFeatureRequestType[i]);
                return (UNEXPECTED_ERROR);
        }
    }
    
    /********************************************************************************/
    /* dynamicRangeControlParameterInterface */
    /********************************************************************************/
    
    pUniDrcSelProcParams->boost    = dynamicRangeControlParameterInterface->boost; /* linear factor for positive gains [dB] */
    pUniDrcSelProcParams->compress = dynamicRangeControlParameterInterface->compress; /* linear factor for negative gains [dB] */
    pUniDrcSelProcParams->drcCharacteristicTarget = dynamicRangeControlParameterInterface->drcCharacteristicTarget;
    
    /********************************************************************************/
    /* MPEG-H parameters */
    /********************************************************************************/

#if MPEG_H_SYNTAX
    pUniDrcSelProcParams->numGroupIdsRequested = mpeghParameterInterface->numGroupIdsRequested;
    for (i=0; i<mpeghParameterInterface->numGroupIdsRequested; i++) {
        pUniDrcSelProcParams->groupIdRequested[i] = mpeghParameterInterface->groupIdRequested[i];
    }
    pUniDrcSelProcParams->numGroupPresetIdsRequested = mpeghParameterInterface->numGroupPresetIdsRequested;
    for (i=0; i<mpeghParameterInterface->numGroupPresetIdsRequested; i++) {
        pUniDrcSelProcParams->groupPresetIdRequested[i] = mpeghParameterInterface->groupPresetIdRequested[i];
        pUniDrcSelProcParams->numMembersGroupPresetIdsRequested[i] = mpeghParameterInterface->numMembersGroupPresetIdsRequested[i];
    }
    pUniDrcSelProcParams->groupPresetIdRequestedPreference = mpeghParameterInterface->groupPresetIdRequestedPreference;
#endif
    
    /********************************************************************************/
    /* EQ parameters */
    /********************************************************************************/
    
#if MPEG_D_DRC_EXTENSION_V1
    pUniDrcSelProcParams->loudnessEqRequest = loudnessEqParameterInterface->loudnessEqRequest;
    pUniDrcSelProcParams->sensitivity = loudnessEqParameterInterface->sensitivity;
    pUniDrcSelProcParams->playbackGain = loudnessEqParameterInterface->playbackGain;
    pUniDrcSelProcParams->eqSetPurposeRequest = equalizationControlInterface->eqSetPurposeRequest;
#endif
    
    return (0);
}

int
evaluateCustomParams_selectionProcess(UniDrcSelProcParams* pUniDrcSelProcParams)
{
    pUniDrcSelProcParams->loudnessNormalizationGainDbMax = max (0.0f, pUniDrcSelProcParams->loudnessNormalizationGainDbMax);
    pUniDrcSelProcParams->loudnessDeviationMax = max (0, pUniDrcSelProcParams->loudnessDeviationMax);
    
    return (0);
}
    
#if MPEG_D_DRC_EXTENSION_V1
int
setLoudEqParameters_uniDrcInterface(UniDrcSelProcParams* pUniDrcSelProcParams, HANDLE_UNI_DRC_INTERFACE hUniDrcInterface)
{
    
    hUniDrcInterface->uniDrcInterfaceExtensionPresent = 1;
    hUniDrcInterface->uniDrcInterfaceExtension.loudnessEqParameterInterfacePresent = 1;
    
    if (pUniDrcSelProcParams->loudnessEqRequest != LOUD_EQ_REQUEST_OFF) {
        hUniDrcInterface->uniDrcInterfaceExtension.loudnessEqParameterInterface.loudnessEqRequestPresent = 1;
        hUniDrcInterface->uniDrcInterfaceExtension.loudnessEqParameterInterface.loudnessEqRequest = pUniDrcSelProcParams->loudnessEqRequest;
    } else {
        hUniDrcInterface->uniDrcInterfaceExtension.loudnessEqParameterInterface.loudnessEqRequestPresent = 0;
        hUniDrcInterface->uniDrcInterfaceExtension.loudnessEqParameterInterface.loudnessEqRequest = LOUD_EQ_REQUEST_OFF;
    }

    hUniDrcInterface->uniDrcInterfaceExtension.loudnessEqParameterInterface.playbackGainPresent = 1;
    hUniDrcInterface->uniDrcInterfaceExtension.loudnessEqParameterInterface.playbackGain        = pUniDrcSelProcParams->playbackGain;
    hUniDrcInterface->uniDrcInterfaceExtension.loudnessEqParameterInterface.sensitivityPresent  = 1;
    hUniDrcInterface->uniDrcInterfaceExtension.loudnessEqParameterInterface.sensitivity         = pUniDrcSelProcParams->sensitivity;
    
    return (0);
}
#endif
    
#ifdef __cplusplus
}
#endif /* __cplusplus */

